/*
 * vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker:
 * channel.c
 *
 * Copyright (C) Keyphrene.com 2005, All rights reserved
 *
 *
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA


 */
#include <Python.h>
#define SSH2_MODULE
#include "ssh2.h"

static char SSH2_Channel_close_doc[] = "";

static PyObject *
SSH2_Channel_close(SSH2_ChannelObj *self, PyObject *args)
{
    int ret;
    
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    ret = libssh2_channel_close(self->channel);
    MY_END_ALLOW_THREADS(self->tstate);
    
    if (ret) {
        PyErr_SetString(PySSH2_Error, "Unable to close the channel");
        return NULL;
    }
    return PyInt_FromLong(1);
}

static char SSH2_Channel_pty_doc[] = "";

static PyObject *
SSH2_Channel_pty(SSH2_ChannelObj *self, PyObject *args)
{
    int ret, lt;
    char *term;
    char *modes = NULL;
    int lm = 0;
    int w = 80;
    int h = 24;
    int pw = 0;
    int ph = 0;
    
    
    if (!PyArg_ParseTuple(args, "s#|s#iiii:pty", &term, &lt, &modes, &lm, &w, &h, &pw, &ph))
        return NULL;
    
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    ret = libssh2_channel_request_pty_ex(self->channel, term, lt, modes, lm, w, h, pw, ph);
    MY_END_ALLOW_THREADS(self->tstate);
    
    if (ret) {
        PyErr_SetString(PySSH2_Error, "Failed requesting pty.");
        return NULL;
    }
    return PyInt_FromLong(1);
}

static char SSH2_Channel_shell_doc[] = "";

static PyObject *
SSH2_Channel_shell(SSH2_ChannelObj *self, PyObject *args)
{    
    int ret;
    
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    ret = libssh2_channel_shell(self->channel);
    MY_END_ALLOW_THREADS(self->tstate);
    
    if (ret) {
        PyErr_SetString(PySSH2_Error, "Unable to request shell on allocated pty.");
        return NULL;
    }
    return PyInt_FromLong(1);
}

static char SSH2_Channel_execute_doc[] = "";

static PyObject *
SSH2_Channel_execute(SSH2_ChannelObj *self, PyObject *args)
{    
    char *cmd;
    int ret;
    
    if (!PyArg_ParseTuple(args, "s:execute", &cmd))
        return NULL;
    
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    ret = libssh2_channel_exec(self->channel, cmd);
    MY_END_ALLOW_THREADS(self->tstate);
    
    if (ret) {
        PyErr_SetString(PySSH2_Error, "Unable to request exec command.");
        return NULL;
    }
    return PyInt_FromLong(1);
}


static char SSH2_Channel_setEnv_doc[] = "";

static PyObject *
SSH2_Channel_setEnv(SSH2_ChannelObj *self, PyObject *args)
{    
    char *key;
    char *val;
    int ret;
    
    if (!PyArg_ParseTuple(args, "ss:setEnv", &key, &val))
        return NULL;
    
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    ret = libssh2_channel_setenv(self->channel, key, val);
    MY_END_ALLOW_THREADS(self->tstate);
    
    if (ret == -1) {
        PyErr_SetString(PySSH2_Error, "Unable to set environment variable.");
        return NULL;
    }
    return PyInt_FromLong(1);
}

static char SSH2_Channel_setBlocking_doc[] = "Set whatever channel is blocking or not\n\
args:\n\
    self, blocking value\n\
    blocking value:\n\
    0 for non-blocking mode\n\
    1 for blocking mode\n\
";

static PyObject *
SSH2_Channel_setBlocking(SSH2_ChannelObj *self, PyObject *args)
{    
    int b=1;
    
    if (!PyArg_ParseTuple(args, "i:setBlocking", &b))
        return NULL;
    /*
    if (b == 1)
    {
        libssh2_channel_handle_extended_data2(self->channel, LIBSSH2_CHANNEL_EXTENDED_DATA_NORMAL);
    } 
    else
    {
        libssh2_channel_handle_extended_data(self->channel, LIBSSH2_CHANNEL_EXTENDED_DATA_NORMAL);
    }
    */

    libssh2_channel_set_blocking(self->channel, b);
    Py_INCREF(Py_None);
    return Py_None;
}

static char SSH2_Channel_read_doc[] = "";

static PyObject *
SSH2_Channel_read(SSH2_ChannelObj *self, PyObject *args)
{    
    int bufsiz, ret=0;
    //~ unsigned char *buf[1024];
    PyObject *buf;
    
    if (!PyArg_ParseTuple(args, "i:read", &bufsiz))
        return NULL;
    
    buf = PyString_FromStringAndSize(NULL, bufsiz);
    if (buf == NULL)
        return NULL;
    
    if (libssh2_channel_eof(self->channel)!=1)
    {
        
        MY_BEGIN_ALLOW_THREADS(self->tstate);
        ret = libssh2_channel_read(self->channel, PyString_AsString(buf), bufsiz);
        MY_END_ALLOW_THREADS(self->tstate);
        
        if (ret > 0)
        {
            if (ret != bufsiz && _PyString_Resize(&buf, ret) < 0)
            {
                return NULL;
            }
            return buf;
        }
    }
    
    Py_DECREF(buf);
    Py_INCREF(Py_None);
    return Py_None;
}

static char SSH2_Channel_readstderr_doc[] = "";

static PyObject *
SSH2_Channel_readstderr(SSH2_ChannelObj *self, PyObject *args)
{    
    int bufsiz, ret=0;
    //~ unsigned char *buf[1024];
    PyObject *buf;
    
    if (!PyArg_ParseTuple(args, "i:read", &bufsiz))
        return NULL;
    
    buf = PyString_FromStringAndSize(NULL, bufsiz);
        if (buf == NULL)
            return NULL;
    
    if (libssh2_channel_eof(self->channel)!=1)
    {
        
        MY_BEGIN_ALLOW_THREADS(self->tstate);
        ret = libssh2_channel_read_stderr(self->channel, PyString_AsString(buf), bufsiz);
        MY_END_ALLOW_THREADS(self->tstate);
        
        if (ret > 0) 
        {
            if (ret != bufsiz && _PyString_Resize(&buf, ret) < 0)
            {
                return NULL;
            }
            return buf;
        }
    }
    
    Py_DECREF(buf);
    Py_INCREF(Py_None);
    return Py_None;
}


static char SSH2_Channel_write_doc[] = "";

static PyObject *
SSH2_Channel_write(SSH2_ChannelObj *self, PyObject *args)
{    
    unsigned char *msg;
    int len;
    int ret=0;
    
    if (!PyArg_ParseTuple(args, "s#:write", &msg, &len))
        return NULL;
    
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    ret = libssh2_channel_write(self->channel, msg, len);
    MY_END_ALLOW_THREADS(self->tstate);
    
    if (ret == -1) {
        PyErr_SetString(PySSH2_Error, "Unable to write.");
        return NULL;
    }
    return PyInt_FromLong(ret);
}

static char SSH2_Channel_flush_doc[] = "";

static PyObject *
SSH2_Channel_flush(SSH2_ChannelObj *self, PyObject *args)
{    
    int ret=0;
    
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    ret = libssh2_channel_flush(self->channel);
    MY_END_ALLOW_THREADS(self->tstate);
    if ( ret == -1) {
        PyErr_SetString(PySSH2_Error, "Unable to flush.");
        return NULL;
    }
    return PyInt_FromLong(ret);
}

static char SSH2_Channel_eof_doc[] = "";

static PyObject *
SSH2_Channel_eof(SSH2_ChannelObj *self, PyObject *args)
{    
    return PyInt_FromLong(libssh2_channel_eof(self->channel));
}

static char SSH2_Channel_sendEof_doc[] = "";

static PyObject *
SSH2_Channel_sendEof(SSH2_ChannelObj *self, PyObject *args)
{    
    int ret=0;
    
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    ret = libssh2_channel_send_eof(self->channel);
    MY_END_ALLOW_THREADS(self->tstate);
    
    if (ret == -1) {
        PyErr_SetString(PySSH2_Error, "Unable to send an EOF.");
        return NULL;
    }
    return PyInt_FromLong(ret);
}

static char SSH2_Channel_windowRead_doc[] = "";

static PyObject *
SSH2_Channel_windowRead(SSH2_ChannelObj *self, PyObject *args)
{    
    unsigned long ret=0;
    unsigned long read_avail;
    unsigned long window_size_initial;
    
    if (!PyArg_ParseTuple(args, "|ii:windowRead", &read_avail, &window_size_initial))
        return NULL;
    
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    ret = libssh2_channel_window_read_ex(self->channel, &read_avail, &window_size_initial);
    MY_END_ALLOW_THREADS(self->tstate);
    
    
    // printf("%d %d", ret, read_avail);
    return PyInt_FromLong(ret);
}

 /* SSH2_Channel_pollRead
 * Returns 0 if no data is waiting on channel,
 * non-0 if data is available */
static char SSH2_Channel_pollRead_doc[] = "";

static PyObject *
SSH2_Channel_pollRead(SSH2_ChannelObj *self, PyObject *args)
{    
    int ret=0;
    int ext=0;
    
    if (!PyArg_ParseTuple(args, "|i:pollRead", &ext))
        return NULL;
    
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    ret = libssh2_poll_channel_read(self->channel, ext);
    MY_END_ALLOW_THREADS(self->tstate);

    return PyInt_FromLong(ret);
}

static char SSH2_Channel_poll_doc[] = "";

static PyObject *
SSH2_Channel_poll(SSH2_ChannelObj *self, PyObject *args)
{    
    int count=2;
    int timeout=0;
    int changed = 0;
    int i;
    LIBSSH2_POLLFD *pollfd;
    PyObject *events;
    
    if (!PyArg_ParseTuple(args, "iO:poll", &timeout, &events))
        return NULL;

    if (!PyList_Check(events))
        return NULL;
    
    count = PyList_Size(events);

    if (!(pollfd = PyMem_Malloc(sizeof(LIBSSH2_POLLFD) * (count + 1))))
    {
        return NULL;
    }
    
    for (i = 0; i<count ; i++)
    {
        
        pollfd[i].type = LIBSSH2_POLLFD_CHANNEL;
        pollfd[i].fd.channel = self->channel;
        pollfd[i].events = PyInt_AsLong(PyList_GetItem(events, i));
        pollfd[i].revents = 0;
    }
    MY_BEGIN_ALLOW_THREADS(self->tstate);
    changed = libssh2_poll(pollfd, count, timeout);
    MY_END_ALLOW_THREADS(self->tstate);
    
    PyMem_Free(pollfd);

    return PyInt_FromLong(changed);
}



/*
 * ADD_METHOD(name) expands to a correct PyMethodDef declaration
 *   {  'name', (PyCFunction)SSH2_Channel_name, METH_VARARGS }
 * for convenience
 */
#define ADD_METHOD(name) { #name, (PyCFunction)SSH2_Channel_##name, METH_VARARGS, SSH2_Channel_##name##_doc }
static PyMethodDef SSH2_Channel_methods[] =
{
    ADD_METHOD(close),
    ADD_METHOD(pty),
    ADD_METHOD(shell),
    ADD_METHOD(execute),
    ADD_METHOD(setEnv),
    ADD_METHOD(setBlocking),
    ADD_METHOD(read),
    ADD_METHOD(readstderr),
    ADD_METHOD(write),
    ADD_METHOD(flush),
    ADD_METHOD(eof),
    ADD_METHOD(sendEof),
    ADD_METHOD(windowRead),
    ADD_METHOD(pollRead),
    ADD_METHOD(poll),
    { NULL, NULL }
};
#undef ADD_METHOD


/*
 * Constructor for Channel objects, never called by Python code directly
 *
 * Arguments: cert    - A "real" Channel certificate object
 *            dealloc - Boolean value to specify whether the destructor should
 *                      free the "real" Channel object
 * Returns:   The newly created Channel object
 */
SSH2_ChannelObj *
SSH2_Channel_New(LIBSSH2_CHANNEL *channel, int dealloc)
{
    SSH2_ChannelObj *self;

    self = PyObject_New(SSH2_ChannelObj, &SSH2_Channel_Type);

    if (self == NULL)
        return NULL;

    self->channel = channel;
    self->tstate = NULL;
    self->dealloc = dealloc;

    return self;
}

/*
 * Deallocate the memory used by the Channel object
 *
 * Arguments: self - The Channel object
 * Returns:   None
 */
static void
SSH2_Channel_dealloc(SSH2_ChannelObj *self)
{
    //~ if (self->dealloc)
        //~ libssh2_channel_free(self->channel);

    PyObject_Del(self);
}

/*
 * Find attribute
 *
 * Arguments: self - The Channel object
 *            name - The attribute name
 * Returns:   A Python object for the attribute, or NULL if something went
 *            wrong
 */
static PyObject *
SSH2_Channel_getattr(SSH2_ChannelObj *self, char *name)
{
    return Py_FindMethod(SSH2_Channel_methods, (PyObject *)self, name);
}

PyTypeObject SSH2_Channel_Type = {
    PyObject_HEAD_INIT(NULL)
    0,
    "Channel",
    sizeof(SSH2_ChannelObj),
    0,
    (destructor)SSH2_Channel_dealloc,
    NULL, /* print */
    (getattrfunc)SSH2_Channel_getattr,
    NULL, /* setattr */
    NULL, /* compare */
    NULL, /* repr */
    NULL, /* as_number */
    NULL, /* as_sequence */
    NULL, /* as_mapping */
    NULL, /* hash */
};

/*
 * Initialize the Channel
 *
 * Arguments: dict - The SSH2 module dictionary
 * Returns:   None
 */
int
init_SSH2_Channel(PyObject *dict)
{
    SSH2_Channel_Type.ob_type = &PyType_Type;
    Py_INCREF(&SSH2_Channel_Type);
    PyDict_SetItemString(dict, "ChannelType", (PyObject *)&SSH2_Channel_Type);
    return 1;
}

