/*
 * vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker:
 *
 * ssh.c
 *
 * Copyright (C) Keyphrene.com 2005, All rights reserved
 *
 *
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA


 */
#include <Python.h>
#define SSH2_MODULE
#include "ssh2.h"

static char SSH2_doc[] = "\n\
";


PyObject *PySSH2_Error;

static char SSH2_Session_doc[] = "\n\
generate a session\n\
\n\
Arguments: spam - Always NULL\n\
           i - dealloc\n\
Returns:   A Session\n\
";


static PyObject *
SSH2_Session(PyObject *spam, PyObject *args)
{    
    int dealloc = 1;
    
    if (!PyArg_ParseTuple(args, "|i:Session", &dealloc))
        return NULL;

    return (PyObject *)SSH2_Session_New(libssh2_session_init(), dealloc);
}


static char SSH2_Channel_doc[] = "\n\
generate a session\n\
\n\
Arguments: spam - Always NULL\n\
           i - dealloc\n\
Returns:   A Session\n\
";


static PyObject *
SSH2_Channel(PyObject *spam, PyObject *args)
{    
    SSH2_SessionObj *session;
    int dealloc = 1;
    
    if (!PyArg_ParseTuple(args, "O|i:Channel", &session, &dealloc))
        return NULL;

    return (PyObject *)SSH2_Channel_New(libssh2_channel_open_session(session->session), dealloc);
}

static char SSH2_SFTP_doc[] = "";
static PyObject *
SSH2_SFTP(PyObject *spam, PyObject *args)
{    
    SSH2_SessionObj *session;
    int dealloc = 1;
    
    if (!PyArg_ParseTuple(args, "O|i:SFTP", &session, &dealloc))
        return NULL;

    return (PyObject *)SSH2_SFTP_New(libssh2_sftp_init(session->session), dealloc);
}

/* Methods in the OpenSSL.ssh module (i.e. none) */
static PyMethodDef SSH2_methods[] = {
    /* Module functions */
    { "Session",(PyCFunction)SSH2_Session,   METH_VARARGS, SSH2_Session_doc },
    { "Channel",(PyCFunction)SSH2_Channel,   METH_VARARGS, SSH2_Channel_doc },
    { "SFTP",(PyCFunction)SSH2_SFTP,   METH_VARARGS, SSH2_SFTP_doc },
    { NULL, NULL }
};

/*
 * Initialize ssh sub module
 *
 * Arguments: None
 * Returns:   None
 */
void
initSSH2(void)
{
    static void *SSH2_API[SSH2_API_pointers];
    PyObject *c_api_object;
    PyObject *module, *dict;

    //~ ERR_load_SSH2_strings();
    //~ OpenSSL_add_all_algorithms();

    if ((module = Py_InitModule3("SSH2", SSH2_methods, SSH2_doc)) == NULL)
        return;

    /* Initialize the C API pointer array */
    SSH2_API[SSH2_Session_New_NUM]      = (void *)SSH2_Session_New;
    SSH2_API[SSH2_Channel_New_NUM]      = (void *)SSH2_Channel_New;
    SSH2_API[SSH2_SFTP_New_NUM]      = (void *)SSH2_SFTP_New;
    SSH2_API[SSH2_SFTP_handle_New_NUM]      = (void *)SSH2_SFTP_handle_New;
    c_api_object = PyCObject_FromVoidPtr((void *)SSH2_API, NULL);
    if (c_api_object != NULL)
        PyModule_AddObject(module, "_C_API", c_api_object);

    PySSH2_Error = PyErr_NewException("SSH2.Error", NULL, NULL);
    if (PySSH2_Error == NULL)
        goto error;
    if (PyModule_AddObject(module, "Error", PySSH2_Error) != 0)
        goto error;


    PyModule_AddStringConstant(module, "__version__",  PACKAGE_VERSION);

    PyModule_AddIntConstant(module, "FINGERPRINT_SHA1",  LIBSSH2_HOSTKEY_HASH_SHA1);

    PyModule_AddIntConstant(module, "FINGERPRINT_MD5",  LIBSSH2_HOSTKEY_HASH_MD5);
   /* PyModule_AddIntConstant(module, "FINGERPRINT_MD5",  0x0000);
    PyModule_AddIntConstant(module, "FINGERPRINT_SHA1",  0x0001);
    PyModule_AddIntConstant(module, "FINGERPRINT_HEX",  0x0000);
    PyModule_AddIntConstant(module, "FINGERPRINT_RAW",  0x0002); */
    
    // methods
    PyModule_AddIntConstant(module, "METHOD_KEX",  LIBSSH2_METHOD_KEX);
    PyModule_AddIntConstant(module, "METHOD_HOSTKEY",  LIBSSH2_METHOD_HOSTKEY);
    PyModule_AddIntConstant(module, "METHOD_CRYPT_CS",  LIBSSH2_METHOD_CRYPT_CS);
    PyModule_AddIntConstant(module, "METHOD_CRYPT_SC",  LIBSSH2_METHOD_CRYPT_SC);
    PyModule_AddIntConstant(module, "METHOD_MAC_CS",  LIBSSH2_METHOD_MAC_CS);
    PyModule_AddIntConstant(module, "METHOD_MAC_SC",  LIBSSH2_METHOD_MAC_SC);
    PyModule_AddIntConstant(module, "METHOD_COMP_CS",  LIBSSH2_METHOD_COMP_CS);
    PyModule_AddIntConstant(module, "METHOD_COMP_SC",  LIBSSH2_METHOD_COMP_SC);
    
    PyModule_AddIntConstant(module, "SFTP_SYMLINK",  LIBSSH2_SFTP_SYMLINK);
    PyModule_AddIntConstant(module, "SFTP_READLINK",  LIBSSH2_SFTP_READLINK);
    PyModule_AddIntConstant(module, "SFTP_REALPATH",  LIBSSH2_SFTP_REALPATH);
    
    PyModule_AddIntConstant(module, "SFTP_STAT",  LIBSSH2_SFTP_STAT);
    PyModule_AddIntConstant(module, "SFTP_LSTAT",  LIBSSH2_SFTP_LSTAT);
    
    PyModule_AddStringConstant(module, "DEFAULT_BANNER",  LIBSSH2_SSH_DEFAULT_BANNER);
    PyModule_AddStringConstant(module, "LIBSSH2_VERSION",  LIBSSH2_VERSION);

    /* poll constants */
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_POLLIN",  LIBSSH2_POLLFD_POLLIN);
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_POLLPRI", LIBSSH2_POLLFD_POLLPRI);
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_POLLEXT", LIBSSH2_POLLFD_POLLEXT);
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_POLLOUT", LIBSSH2_POLLFD_POLLOUT);
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_POLLERR", LIBSSH2_POLLFD_POLLERR);
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_POLLHUP", LIBSSH2_POLLFD_POLLHUP);
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_SESSION_CLOSED", LIBSSH2_POLLFD_SESSION_CLOSED);
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_POLLNVAL", LIBSSH2_POLLFD_POLLNVAL);
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_POLLEX", LIBSSH2_POLLFD_POLLEX);
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_CHANNEL_CLOSED", LIBSSH2_POLLFD_CHANNEL_CLOSED);
    PyModule_AddIntConstant(module, "LIBSSH2_POLLFD_LISTENER_CLOSED", LIBSSH2_POLLFD_LISTENER_CLOSED);

    dict = PyModule_GetDict(module);
    if (!init_SSH2_Session(dict))
        goto error;
    if (!init_SSH2_Channel(dict))
        goto error;
    if (!init_SSH2_SFTP(dict))
        goto error;
    if (!init_SSH2_SFTP_handle(dict))
        goto error;
error:
    ;
}

