#!/usr/bin/env python
# vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker:

import os, sys, socket, time
import unittest, unittest_env

import SSH2

class SSH2SFTPUnitTest(unittest.TestCase):
    
    def getConnexion(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((HOSTNAME, PORT))
        sock.setblocking(1)
        return sock
    
    def testReadDir(self):
        print "########\n#\n# test SFTP: ReadDir...\n#\n########"
        ssh = SSH2.Session()
        ssh.setBanner(SSH2.DEFAULT_BANNER)
        ssh.setMethod(SSH2.METHOD_COMP_SC, "zlib,none")
        ssh.setMethod(SSH2.METHOD_COMP_CS, "zlib,none")
        ssh.startup(self.getConnexion())
        ssh.setPassword(LOGIN, PWD)
        sftp = ssh.SFTP()
        try:
            handle = sftp.openDir(PATH)
            if handle:
                while 1:
                    data = sftp.readDir(handle)
                    if not data: break
                    print data
            sftp.close(handle)
        except SSH2.Error, e:
            print "*********** SSH2 ERROR *********\n%s\n******************" %e
        sftp.shutdown()
        ssh.close()
    
    def testListDir(self):
        print "########\n#\n# test SFTP: ListDir...\n#\n########"
        ssh = SSH2.Session()
        #ssh.setMethod(SSH2.METHOD_COMP_CS, "zlib")
        ssh.setBanner(SSH2.DEFAULT_BANNER)
        ssh.startup(self.getConnexion())
        ssh.setPassword(LOGIN, PWD)
        sftp = ssh.SFTP()
        try:
            handle = sftp.openDir(PATH)
            if handle:
                for f, attr in sftp.listDir(handle):
                    print f, attr
                sftp.close(handle)
        except SSH2.Error, e:
            print "*********** SSH2 ERROR *********\n%s\n******************" %e
        sftp.shutdown()
        ssh.setDebug()
        ssh.close()
        
    def testSFTPRead(self):
        print "########\n#\n# test SFTP: Read ...\n#\n########"
        
        filename = os.path.basename(FILENAMETODOWNLOAD)
        if os.path.isfile(filename): os.unlink(filename)
        ssh = SSH2.Session()
        ssh.setBanner(SSH2.DEFAULT_BANNER)
        ssh.setMethod(SSH2.METHOD_COMP_CS, "zlib,none")
        print "startup"
        ssh.startup(self.getConnexion())
        ssh.setPassword(LOGIN, PWD)
        
        sftp = ssh.SFTP()
        try:
            stat = sftp.getStat(FILENAMETODOWNLOAD)
            handle = sftp.open(FILENAMETODOWNLOAD)
            if handle:
                t = time.time()
                #f = open(filename, "wb")
                f = open(filename, "wb")
                while 1:
                    data = sftp.read(handle, BUFFER)
                    if not data: break
                    f.write(data)
                sftp.close(handle)
                f.close()
                print "SFTP: Read: %d KB/s" % ((stat[0]/1000)/(time.time()-t))
            print "Downloaded file size: %d" %os.stat(filename).st_size
        except SSH2.Error, e:
            print "*********** SSH2 ERROR *********\n%s\n******************" %e
        sftp.shutdown()
            
    def testSCPRead(self):
        print "########\n#\n# test SCP: Read ...\n#\n########"

        filename = os.path.basename(FILENAMETODOWNLOAD)
        if os.path.isfile(filename): os.unlink(filename)
        ssh = SSH2.Session()
        ssh.setBanner(SSH2.DEFAULT_BANNER)
        ssh.setMethod(SSH2.METHOD_COMP_CS, "zlib,none")
        ssh.startup(self.getConnexion())
        ssh.setPassword(LOGIN, PWD)
        try:                
            channel, stats = ssh.SCPGet(FILENAMETODOWNLOAD)
            size = stats[0]
            print "Remote file size: " + str(size)
            print stats
            if channel:
                channel.setBlocking(False)
                t = time.time()
                f = open(filename, "wb")
                B = BUFFER
                l = 0
                #print SSH2.POLLIN,SSH2.POLLPRI
                while 1:
                    if channel.eof():
                         break
                    if not channel.poll(5000,[SSH2.POLLIN,SSH2.POLLPRI]):
                        print "no data"
                        raise Exception('File incomplete')
                    if l +B > size:
                        B = size-l
                        #if B == 0: break
                    data = channel.read(B) # BUF represente la quantite maximum de donnees, mais on recuperer moins
                    if not data: break
                    f.write(data)
                    l +=len(data)
                #channel.close()
                f.close()
            print "SCP: Read: %d KB/s" % ((stats[0]/1000)/(time.time()-t))
            print "Local file size: %d" %os.stat(filename).st_size
        except SSH2.Error, e:
            print "*********** SSH2 ERROR *********\n%s\n******************" %e
        ssh.close() 
            
    def testSFTPwrite(self):
        print "########\n#\n# test  SFTP: Write...\n#\n########"

        filename = os.path.basename(FILENAMETODOWNLOAD)
        ssh = SSH2.Session()
        ssh.setBanner(SSH2.DEFAULT_BANNER)
        ssh.setMethod(SSH2.METHOD_COMP_CS, "zlib,none")
        ssh.startup(self.getConnexion())
        ssh.setPassword(LOGIN, PWD)
        sftp = ssh.SFTP()
        try:

            stat = os.stat(filename)
            handle = sftp.open(FILENAMETODOWNLOAD+"2", "w")
            if handle:
                t = time.time()
                f = open(filename, "rb")
                while 1:
                    data = f.read(BUFFER)
                    if not data: break
                    sftp.write(handle, data)
                sftp.close(handle)
                f.close()
                print "SFTP: Write: %d KB/s" % ((stat.st_size/1000)/(time.time()-t))
        except SSH2.Error, e:
            print "*********** SSH2 ERROR *********\n%s\n******************" %e
        except os.error, e:
            print "*********** OS ERROR *********\n%s\n******************" %e

        sftp.shutdown()
        
                
    def testSCPwrite(self):
        print "########\n#\n# test  SCP: Write...\n#\n########"

        filename = os.path.basename(FILENAMETODOWNLOAD)
        ssh = SSH2.Session()
        ssh.setBanner(SSH2.DEFAULT_BANNER+" Python (http://www.keyphrene.com/projects/org.keyphrene)")
        #ssh.setMethod(SSH2.METHOD_COMP_CS, "zlib")
        print "startup"
        ssh.startup(self.getConnexion())
        ssh.setPassword(LOGIN, PWD)
        try:
            stat = os.stat(filename)
            size = stat.st_size
            channel = ssh.SCPPut(FILENAMETODOWNLOAD+"3", 0744, size)
            if channel:
                channel.setBlocking(True)
                t = time.time()
                f = open(filename, "rb")
                B = BUFFER
                while 1:
                    buf = f.read(B)
                    if not buf: break
                    l = 0
                    while buf!="":
                        r = channel.write(buf)
                        l+=r
                        if l >= B or r==0:
                            break
                        else:
                            buf = buf[r:]
                f.close()
                channel.close()
                print "SCP: Write: %d KB/s" % ((size/1000)/(time.time()-t))
        except SSH2.Error, e:
            print "*********** SSH2 ERROR *********\n%s\n******************" %e
        except os.error, e:
            print "*********** OS ERROR *********\n%s\n******************" %e
        ssh.close()



if __name__ == "__main__": 
    print
    print "UnitTest: SSH2, SFTP, SCP"
    print "Keyphrene.com - 2005 - All rights reserved"
    HOSTNAME = "127.0.0.1"
    PORT = 22
    LOGIN = "tom"
    PWD = "********"
    PATH = "/home/tom"
    #PATH = "/home/tom/xulu"
    FILENAMETODOWNLOAD = "/home/tom/mbox"
    #FILENAMETODOWNLOAD = "/home/tom/xulu"
    BUFFER = 8192
    
    
    unittest.main()
